package jdbc.example.employee.dao;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.PropertySource;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCallback;
import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.stereotype.Repository;

import jdbc.example.dependant.bean.Dependant;
import jdbc.example.dependant.bean.Dependants;
import jdbc.example.employee.bean.Employee;
import jdbc.example.hospilalisation.bean.Hospitalisation;
import jdbc.example.hospilalisation.bean.Hospitalisations;

@Repository("employeeDAO")

public class EmployeeDAO {
	private static Logger log = LoggerFactory.getLogger(EmployeeDAO.class);
	@Autowired
	private JdbcTemplate jdbcTemplate;

	public void createEmployee(Employee employee) throws SQLException, DataAccessException {
		log.debug("createRecord start");
		log.debug("createRecord start");
		String createEmployeeQuery = "INSERT INTO EMPLOYEE(EMPLOYEE_KEY, EMP_ID, NAME, DEPT_CODE, BASIC_SAL, DATE_OF_JOINING, ADDRESS) VALUES ( ?,?,?,?,?,?,?)";
		jdbcTemplate.execute(createEmployeeQuery, new PreparedStatementCallback<Object>() {
			@Override
			public Object doInPreparedStatement(PreparedStatement ps) throws SQLException, DataAccessException {
				int indx = 1;

				employee.setEmployeeKey(System.currentTimeMillis());
				ps.setLong(indx++, employee.getEmployeeKey());
				ps.setString(indx++, employee.getEmpId());
				ps.setString(indx++, employee.getName());
				ps.setString(indx++, employee.getDeptCode());
				ps.setDouble(indx++, employee.getBasicSal());
				ps.setDate(indx++, employee.getDateOfJoining());
				ps.setString(indx++, employee.getAddress());

				
				return ps.executeUpdate();
			}
		});
	}

	

	
	

	public List<Employee> getEmplist(Employee employee) {
		
		String getEmplistApiQuery="" 
				+"SELECT"
				+"   EMPLOYEE.*, "
				+"   DEPENDANT.*,"
				+"   HOSPITALISATION.*"
				+"FROM"
				+"   EMPLOYEE,"
				+"   DEPENDANT,"
				+"   HOSPITALISATION "
				+"WHERE"
				+"   1 = 1 "
				+"   AND EMPLOYEE.EMP_ID = DEPENDANT.EMP_ID "
				+"   AND DEPENDANT.DEPENDANT_ID = HOSPITALISATION.DEPENDANT_ID "
				+"   AND EMPLOYEE.BASIC_SAL > 100"
				+"";
		
		System.out.println(""+getEmplistApiQuery);
		return jdbcTemplate.query(getEmplistApiQuery, new ResultSetExtractor<List<Employee>>() {
		
			@Override
			public List<Employee> extractData(ResultSet rs) throws SQLException, DataAccessException {

				List<Employee> list = new ArrayList<Employee>();
				Map<Long, Employee> employeeKeyEmployeeMap = new HashMap<Long, Employee>();
				Map<Long, Dependant> dependantKeyDependantMap = new HashMap<Long, Dependant>();
				Map<Long, Hospitalisation> hospitalisationKeyHospitalisationMap = new HashMap<Long, Hospitalisation>();
				while (rs.next()) {

					Long employeeKey = rs.getLong("EMPLOYEE_KEY");
					Employee employee = employeeKeyEmployeeMap.get(employeeKey);
					if (employee == null) {
						employee = new Employee();
						employeeKeyEmployeeMap.put(employeeKey, employee);
						list.add(employee);
						employee.setEmployeeKey(employeeKey);
						log.info("employeeKey=" + employee.getEmployeeKey());
						employee.setAddress(rs.getString("ADDRESS"));
						employee.setBasicSal(rs.getDouble("BASIC_SAL"));
						employee.setDateOfJoining(rs.getDate("DATE_OF_JOINING"));
						employee.setDeptCode(rs.getString("DEPT_CODE"));
						employee.setEmpId(rs.getString("EMP_ID"));
						employee.setEmployeeKey(rs.getLong("EMPLOYEE_KEY"));
						employee.setName(rs.getString("NAME"));
					}
					Long dependantKey = rs.getLong("DEPENDANT_KEY");
					Dependant dependant = dependantKeyDependantMap.get(dependantKey);
					log.info("dependantKeyML=" + dependantKey);
					if (dependant == null) {
						dependant = new Dependant();
						if (employee.getDependants() == null)
							employee.setDependants(new Dependants());
						if (employee.getDependants().getDependantList() == null)
							employee.getDependants().setDependantList(new ArrayList<Dependant>());

						employee.getDependants().getDependantList().add(dependant);
						dependantKeyDependantMap.put(dependantKey, dependant);
						dependant.setDependantKey(dependantKey);
						dependant.setAge(rs.getString("AGE"));
						dependant.setDependantId(rs.getString("DEPENDANT_ID"));
						dependant.setDependantKey(rs.getLong("DEPENDANT_KEY"));
						dependant.setDependantName(rs.getString("DEPENDANT_NAME"));
						dependant.setEmpId(rs.getString("EMP_ID"));
						dependant.setRelationWithEmp(rs.getString("RELATION_WITH_EMP"));
					}
					Long hospitalisationKey = rs.getLong("HOSPITALISATION_KEY");
					Hospitalisation hospitalisation = hospitalisationKeyHospitalisationMap.get(hospitalisationKey);
					log.info("hospitalisationKeyML=" + hospitalisationKey);
					if (hospitalisation == null) {
						hospitalisation = new Hospitalisation();
						if (dependant.getHospitalisations() == null)
							dependant.setHospitalisations(new Hospitalisations());
						if (dependant.getHospitalisations().getHospitalisationList() == null)
							dependant.getHospitalisations().setHospitalisationList(new ArrayList<Hospitalisation>());

						dependant.getHospitalisations().getHospitalisationList().add(hospitalisation);
						hospitalisationKeyHospitalisationMap.put(hospitalisationKey, hospitalisation);
						hospitalisation.setHospitalisationKey(hospitalisationKey);
						hospitalisation.setAdmissionDate(rs.getDate("ADMISSION_DATE"));
						// hospitalisation.setDefault(rs.getString("DEFAULT"));
						hospitalisation.setDependantId(rs.getString("DEPENDANT_ID"));
						hospitalisation.setEmpId(rs.getString("EMP_ID"));
						hospitalisation.setHospitalBillAmt(rs.getDouble("HOSPITAL_BILL_AMT"));
						hospitalisation.setHospitalName(rs.getString("HOSPITAL_NAME"));
						hospitalisation.setHospitalisationKey(rs.getLong("HOSPITALISATION_KEY"));
						hospitalisation.setReleaseDate(rs.getDate("RELEASE_DATE"));
						hospitalisation.setTreatmentDtls(rs.getString("TREATMENT_DTLS"));
					}
				}
				return list;
			}
		});
	}

}
